from graphene_django import DjangoObjectType

from ..models import Room
from .base import (
    BaseBatchCreateMutation,
    BaseBatchDeleteMutation,
    BaseBatchPatchMutation,
    DjangoFilterMixin,
    PermissionsTypeMixin,
)


class RoomType(PermissionsTypeMixin, DjangoFilterMixin, DjangoObjectType):
    class Meta:
        model = Room
        fields = ("id", "name", "short_name")
        filter_fields = {
            "id": ["exact", "lte", "gte"],
            "name": ["icontains"],
            "short_name": ["icontains"],
        }


class RoomBatchCreateMutation(BaseBatchCreateMutation):
    class Meta:
        model = Room
        permissions = ("core.create_room_rule",)
        only_fields = ("id", "name", "short_name")


class RoomBatchDeleteMutation(BaseBatchDeleteMutation):
    class Meta:
        model = Room
        permissions = ("core.delete_room_rule",)


class RoomBatchPatchMutation(BaseBatchPatchMutation):
    class Meta:
        model = Room
        permissions = ("core.edit_room_rule",)
        only_fields = ("id", "name", "short_name")



# Generated by Django 5.0.7 on 2024-07-31 17:06

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models
import recurrence.fields
import timezone_field.fields

class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('core', '0064_rrule_model'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='calendarevent',
            name='timezone_if_datetime_start',
        ),
        migrations.RemoveConstraint(
            model_name='calendarevent',
            name='timezone_if_datetime_end',
        ),
        migrations.AlterField(
            model_name='calendarevent',
            name='recurrences',
            field=recurrence.fields.RecurrenceField(blank=True, default='', verbose_name='Recurrences'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='calendarevent',
            name='timezone',
            field=timezone_field.fields.TimeZoneField(blank=True, default='', verbose_name='Timezone'),
            preserve_default=False,
        ),
        migrations.AddConstraint(
            model_name='calendarevent',
            constraint=models.CheckConstraint(condition=models.Q(('datetime_start__isnull', False), ('timezone', ''),
                                                             models.Q(('recurrences', ''), _negated=True),
                                                             _negated=True),
                                              name='timezone_if_datetime_start_and_recurring'),
        ),
        migrations.AddConstraint(
            model_name='calendarevent',
            constraint=models.CheckConstraint(condition=models.Q(('datetime_end__isnull', False), ('timezone', ''),
                                                             models.Q(('recurrences', ''), _negated=True),
                                                             _negated=True),
                                              name='timezone_if_datetime_end_and_recurring'),
        ),
    ]



import json
from collections.abc import Iterable

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import BadRequest, PermissionDenied
from django.db.models import Model
from django.utils import timezone

import graphene
import reversion
from django_filters.filterset import FilterSet, filterset_factory
from graphene_django import DjangoListField, DjangoObjectType
from graphene_django_cud.mutations.batch_create import DjangoBatchCreateMutation
from graphene_django_cud.mutations.batch_delete import DjangoBatchDeleteMutation
from graphene_django_cud.mutations.batch_patch import DjangoBatchPatchMutation
from reversion import set_comment, set_user

from ..util.core_helpers import queryset_rules_filter


class RulesObjectType(DjangoObjectType):
    class Meta:
        abstract = True

    @classmethod
    def get_queryset(cls, queryset, info, perm):
        q = super().get_queryset(queryset, info)

        return queryset_rules_filter(info.context, q, perm)


class FieldFileType(graphene.ObjectType):
    url = graphene.String()
    absolute_url = graphene.String()
    name = graphene.String()

    def resolve_url(root, info, **kwargs):
        return root.url if root else ""

    def resolve_absolute_url(root, info, **kwargs):
        return info.context.build_absolute_uri(root.url) if root else ""

    def resolve_name(root, info, **kwargs):
        return root.name if root else ""


class DeleteMutation(graphene.Mutation):
    """Mutation to delete an object."""

    klass: Model = None
    permission_required: str = ""
    ok = graphene.Boolean()

    class Arguments:
        id = graphene.ID()  # noqa

    @classmethod
    def mutate(cls, root, info, **kwargs):
        obj = cls.klass.objects.get(pk=kwargs["id"])
        if info.context.user.has_perm(cls.permission_required, obj):
            obj.delete()
            return cls(ok=True)
        else:
            raise PermissionDenied()


class PermissionsTypeMixin:
    """Mixin for adding permissions to a Graphene type.

    To configure the names for the permissions or to do
    different permission checking, override the respective
    methods `resolve_can_edit` and `resolve_can_delete`
    """

    can_edit = graphene.Boolean()
    can_delete = graphene.Boolean()

    @staticmethod
    def resolve_can_edit(root: Model, info, **kwargs):
        if hasattr(root, "managed_by_app_label") and root.managed_by_app_label != "":
            return False
        content_type = ContentType.objects.get_for_model(root)
        perm = f"{content_type.app_label}.edit_{content_type.model}_rule"
        return info.context.user.has_perm(perm, root)

    @staticmethod
    def resolve_can_delete(root: Model, info, **kwargs):
        if hasattr(root, "managed_by_app_label") and root.managed_by_app_label != "":
            return False
        content_type = ContentType.objects.get_for_model(root)
        perm = f"{content_type.app_label}.delete_{content_type.model}_rule"
        return info.context.user.has_perm(perm, root)


class OptimisticResponseTypeMixin:
    """Mixin for using OptimisticResponse in the frontend.

    In the frontend, Apollo can be configured to show an optimistic
    response defined by the developer although the actual request
    isn't yet finished. This way the frontend can directly update
    and replace the optimistc data with the actual data from the
    backend when the request is finished.

    To know whether a response is optimistic or real, the field
    `is_optimistic` can be set to true in the frontend.
    """

    is_optimistic = graphene.Boolean(default_value=False)


class PermissionBatchCreateMixin:
    """Mixin for permission checking during batch create mutations."""

    class Meta:
        login_required = True

    @classmethod
    def check_permissions(cls, root, info, input, *args, **kwargs):  # noqa: A002
        pass

    @classmethod
    def after_create_obj(cls, root, info, data, obj, input):  # noqa
        super().after_create_obj(root, info, data, obj, input)
        if not isinstance(cls._meta.permissions, Iterable) or not info.context.user.has_perms(
            cls._meta.permissions, obj
        ):
            raise PermissionDenied()


class PermissionBatchPatchMixin:
    """Mixin for permission checking during batch patch mutations."""

    class Meta:
        login_required = True

    @classmethod
    def check_permissions(cls, root, info, input, *args, **kwargs):  # noqa: A002
        pass

    @classmethod
    def after_update_obj(cls, root, info, input, obj, full_input):  # noqa
        super().after_update_obj(root, info, input, obj, full_input)
        if (
            hasattr(obj, "managed_by_app_label")
            and obj.managed_by_app_label != ""
            or not isinstance(cls._meta.permissions, Iterable)
            or not info.context.user.has_perms(cls._meta.permissions, obj)
        ):
            raise PermissionDenied()


class PermissionBatchDeleteMixin:
    """Mixin for permission checking during batch delete mutations."""

    class Meta:
        login_required = True

    @classmethod
    def check_permissions(cls, root, info, input, *args, **kwargs):  # noqa: A002
        pass

    @classmethod
    def before_save(cls, root, info, ids, qs_to_delete):  # noqa
        super().before_save(root, info, ids, qs_to_delete)
        if not isinstance(cls._meta.permissions, Iterable):
            raise PermissionDenied()
        for obj in qs_to_delete:
            if (
                hasattr(obj, "managed_by_app_label")
                and obj.managed_by_app_label != ""
                or not info.context.user.has_perms(cls._meta.permissions, obj)
            ):
                raise PermissionDenied()


class PermissionPatchMixin:
    """Mixin for permission checking during patch mutations."""

    class Meta:
        login_required = True

    @classmethod
    def check_permissions(cls, root, info, input, id, obj):  # noqa
        if info.context.user.has_perms(cls._meta.permissions, root):
            return

        raise PermissionDenied()


class DjangoFilterMixin:
    """Filters a queryset with django filter."""

    @classmethod
    def get_filterset(cls):
        meta = getattr(cls, "_meta", None)

        if not meta:
            raise NotImplementedError(f"{cls.__name__} must implement class Meta for filtering.")

        if hasattr(meta, "filterset_class"):
            filterset = meta.filterset_class
            if filterset is not None:
                return filterset

        model: Model = meta.model
        fields = getattr(meta, "filter_fields", None)

        if not model:
            raise NotImplementedError(f"{cls.__name__} must supply a model via the Meta class")

        if not fields:
            # Django filter doesn't allow to filter without explicit fields
            raise NotImplementedError(
                f"{cls.__name__}.Meta must contain filter_fields or a filterset_class"
            )

        fs = filterset_factory(model=model, fields=fields)

        return fs

    @classmethod
    def filter(cls, filters, queryset):  # noqa
        filterset_class = cls.get_filterset()
        filterset: FilterSet = filterset_class(filters, queryset)
        return filterset.qs


class FilterOrderList(DjangoListField):
    """Generic filterable Field for lists of django models."""

    def __init__(self, _type, *args, **kwargs):
        kwargs.update(order_by=graphene.List(graphene.String))
        kwargs.update(filters=graphene.JSONString())
        super().__init__(_type, *args, **kwargs)

    @staticmethod
    def list_resolver(
        django_object_type,
        resolver,
        default_manager,
        root,
        info,
        order_by=None,
        filters=None,
        **args,
    ):
        qs = DjangoListField.list_resolver(
            django_object_type, resolver, default_manager, root, info, **args
        )

        if filters is not None:
            if isinstance(filters, str):
                filters = json.loads(filters)

            if isinstance(filters, dict) and len(filters.keys()) > 0:
                for f_key, f_value in filters.items():
                    if isinstance(f_value, list):
                        filters[f_key] = ",".join(map(str, f_value))

                qs = django_object_type.filter(filters, qs)

        if order_by is not None:
            if isinstance(order_by, str):
                order_by = [order_by]

            qs = qs.order_by(*order_by)

        return qs


class MutateWithRevisionMixin:
    """Mixin for creating revision for mutation."""

    @classmethod
    def mutate(cls, root, info, *args, **kwargs):
        with reversion.create_revision():
            set_user(info.context.user)
            set_comment(cls.__name__)
            return super().mutate(root, info, *args, **kwargs)


class ModelValidationMixin:
    """Mixin for executing model validation mechanisms."""

    @classmethod
    def after_update_obj(cls, root, info, data, obj, full_input):
        super().after_update_obj(root, info, data, obj, full_input)
        obj.full_clean()

    @classmethod
    def before_create_obj(cls, info, data, obj):
        super().before_create_obj(info, data, obj)
        obj.full_clean()


class BaseObjectType(PermissionsTypeMixin, DjangoFilterMixin, DjangoObjectType):
    class Meta:
        abstract = True


class BaseBatchCreateMutation(
    ModelValidationMixin,
    MutateWithRevisionMixin,
    PermissionBatchCreateMixin,
    DjangoBatchCreateMutation,
):
    class Meta:
        abstract = True


class BaseBatchPatchMutation(
    ModelValidationMixin,
    MutateWithRevisionMixin,
    PermissionBatchPatchMixin,
    DjangoBatchPatchMutation,
):
    class Meta:
        abstract = True


class BaseBatchDeleteMutation(
    MutateWithRevisionMixin, PermissionBatchDeleteMixin, DjangoBatchDeleteMutation
):
    class Meta:
        abstract = True


class CalendarEventBatchCreateMixin:
    """Mixin handling the timezone in the BatchCreateMutation of CalendarEvents."""

    @classmethod
    def handle_datetime_start(cls, value, name, info):
        value = value.replace(tzinfo=timezone.get_default_timezone())
        return value

    @classmethod
    def handle_datetime_end(cls, value, name, info):
        value = value.replace(tzinfo=timezone.get_default_timezone())
        return value


class CalendarEventBatchPatchMixin:
    """Mixin handling mutating of CalendarEvents.
    Handles switching between date & datetime.
    Handles removing recurrences.
    """

    @classmethod
    def before_mutate(cls, root, info, input):  # noqa
        super().before_mutate(root, info, input)
        for event in input:
            # Assure either date or datetime is set to None
            if "datetime_start" in event and "datetime_end" in event:
                event["date_start"] = None
                event["date_end"] = None
            elif "date_start" in event and "date_end" in event:
                event["datetime_start"] = None
                event["datetime_end"] = None
            else:
                raise BadRequest(
                    "Set either date_start and date_end or datetime_start and datetime_end"
                )

            # Remove recurrences if none were received.
            if "recurrences" not in event:
                event["recurrences"] = ""

        return input



import copy

from django.apps import apps
from django.contrib.auth import get_user_model
from django.contrib.auth.password_validation import (
    password_validators_help_texts,
    validate_password,
)
from django.contrib.contenttypes.models import ContentType
from django.contrib.messages import get_messages
from django.core.exceptions import ValidationError
from django.db.models import Q

import graphene
import graphene_django_optimizer
from django_celery_results.models import TaskResult
from django_countries import countries
from graphene.types.resolver import dict_or_attr_resolver, set_default_resolver
from guardian.shortcuts import get_objects_for_user
from haystack.inputs import AutoQuery
from haystack.query import SearchQuerySet
from haystack.utils.loading import UnifiedIndex
from health_check.plugins import plugin_dir
from oauth2_provider.generators import generate_client_id, generate_client_secret

from ..celery import app as celery_app
from ..data_checks import DataCheck
from ..models import (
    Announcement,
    AvailabilityEvent,
    AvailabilityType,
    CustomMenu,
    DataCheckResult,
    DynamicRoute,
    Group,
    GroupType,
    Holiday,
    Notification,
    OAuthAccessToken,
    OAuthApplication,
    PDFFile,
    Person,
    PersonInvitation,
    Role,
    Room,
    SchoolTerm,
    TaskUserAssignment,
)
from ..util.apps import AppConfig
from ..util.auth_helpers import AppScopes
from ..util.core_helpers import (
    filter_active_school_term,
    get_active_school_term,
    get_allowed_object_ids,
    get_app_module,
    get_app_packages,
    has_person,
)
from .announcement import (
    AnnouncementBatchCreateMutation,
    AnnouncementBatchDeleteMutation,
    AnnouncementBatchPatchMutation,
    AnnouncementType,
    DisplayAnnouncementType,
)
from .availability_event import (
    AvailabilityEventBatchCreateMutation,
    AvailabilityEventBatchDeleteMutation,
    AvailabilityEventBatchPatchMutation,
    AvailabilityEventType,
    AvailabilityTypeBatchCreateMutation,
    AvailabilityTypeBatchDeleteMutation,
    AvailabilityTypeBatchPatchMutation,
    AvailabilityTypeType,
)
from .base import FilterOrderList
from .calendar import CalendarBaseType, SetCalendarStatusMutation
from .celery_progress import CeleryProgressFetchedMutation, CeleryProgressType, CeleryTaskResultType
from .country import CountryType
from .custom_menu import CustomMenuType
from .dashboard.dashboard import (
    CreateDashboardWidgetsMutation,
    DashboardType,
    DashboardWidgetDeleteMutation,
    DashboardWidgetInstanceDeleteMutation,
    DashboardWidgetUpdateMutation,
    ReorderDashboardWidgetsMutation,
)
from .dashboard.widgets import (
    CalendarWidgetBatchPatchMutation,
    ExternalLinkWidgetBatchPatchMutation,
    StaticContentWidgetBatchPatchMutation,
    dashboard_types,
)
from .data_check import (
    DataCheckResultType,
    DataCheckType,
    RunDataChecksMutation,
    SolveDataCheckResultMutation,
)
from .dynamic_routes import DynamicRouteType
from .group import GroupBatchDeleteMutation
from .group import GroupType as GraphQLGroupType
from .group_type import (
    GroupTypeBatchCreateMutation,
    GroupTypeBatchDeleteMutation,
    GroupTypeBatchPatchMutation,
    GroupTypeType,
)
from .health_check import HealthCheckPluginType
from .holiday import (
    HolidayBatchCreateMutation,
    HolidayBatchDeleteMutation,
    HolidayBatchPatchMutation,
    HolidayType,
)
from .installed_apps import AppType
from .maintenance_mode import SetMaintenanceModeMutation
from .message import MessageType
from .notification import (
    MarkAllNotificationsReadMutation,
    MarkNotificationReadMutation,
    NotificationType,
)
from .oauth import (
    InitOAuthApplicationType,
    OAuthAccessTokenType,
    OAuthApplicationBatchCreateMutation,
    OAuthApplicationBatchDeleteMutation,
    OAuthApplicationBatchPatchMutation,
    OAuthApplicationType,
    OAuthBatchRevokeTokenMutation,
    OAuthScopeType,
)
from .pdf import PDFFileType
from .permissions import ObjectPermissionInputType, ObjectPermissionResultType
from .person import (
    PersonBatchCreateMutation,
    PersonBatchDeleteMutation,
    PersonBatchPatchMutation,
    PersonType,
    SendAccountRegistrationMutation,
)
from .person_invitation import PersonInvitationType
from .personal_event import (
    PersonalEventBatchCreateMutation,
    PersonalEventBatchDeleteMutation,
    PersonalEventBatchPatchMutation,
)
from .personal_todo import (
    PersonalTodoBatchCreateMutation,
    PersonalTodoBatchDeleteMutation,
    PersonalTodoBatchPatchMutation,
)
from .role import RoleBatchCreateMutation, RoleBatchDeleteMutation, RoleBatchPatchMutation, RoleType
from .room import RoomBatchCreateMutation, RoomBatchDeleteMutation, RoomBatchPatchMutation, RoomType
from .school_term import (
    SchoolTermBatchCreateMutation,
    SchoolTermBatchDeleteMutation,
    SchoolTermBatchPatchMutation,
    SchoolTermType,
    SetActiveSchoolTermMutation,
)
from .search import SearchResultType
from .system_properties import SystemPropertiesType
from .todo import SetTodoCompletedMutation
from .two_factor import (
    ActivateTOTPMutation,
    AddSecurityKeyMutation,
    DeactivateAuthenticatorMutation,
    GenerateRecoveryCodesMutation,
    TwoFactorType,
)
from .user import (
    ChangePasswordMutation,
    RequestPasswordResetMutation,
    UserType,
    check_password_reset_key,
)


def custom_default_resolver(attname, default_value, root, info, **args):
    """Custom default resolver to ensure resolvers are set for all queries."""
    if info.parent_type.name == "GlobalQuery":
        raise NotImplementedError(f"No own resolver defined for {attname}")

    return dict_or_attr_resolver(attname, default_value, root, info, **args)


set_default_resolver(custom_default_resolver)


class Query(graphene.ObjectType):
    ping = graphene.String(payload=graphene.String())

    notifications = graphene.List(NotificationType)

    persons = FilterOrderList(PersonType)
    person_by_id = graphene.Field(PersonType, id=graphene.ID())
    person_by_id_or_me = graphene.Field(PersonType, id=graphene.ID())

    groups = FilterOrderList(GraphQLGroupType)
    group_by_id = graphene.Field(GraphQLGroupType, id=graphene.ID())
    groups_by_owner = FilterOrderList(GraphQLGroupType, owner=graphene.ID())

    who_am_i = graphene.Field(UserType)
    users = graphene.List(UserType)

    system_properties = graphene.Field(SystemPropertiesType)
    installed_apps = graphene.List(AppType)

    celery_progress_by_id = graphene.Field(CeleryProgressType, id=graphene.ID())
    celery_progress_by_user = graphene.List(CeleryProgressType)
    celery_inspect_task_results = graphene.List(CeleryTaskResultType)

    pdf_by_id = graphene.Field(PDFFileType, id=graphene.ID())

    search_snippets = graphene.List(
        SearchResultType, query=graphene.String(), limit=graphene.Int(required=False)
    )

    messages = graphene.List(MessageType)

    current_announcements = graphene.List(DisplayAnnouncementType)

    custom_menu_by_name = graphene.Field(CustomMenuType, name=graphene.String())

    dynamic_routes = graphene.List(DynamicRouteType)

    two_factor = graphene.Field(TwoFactorType)

    oauth_access_tokens = graphene.List(OAuthAccessTokenType)

    rooms = FilterOrderList(RoomType)
    room_by_id = graphene.Field(RoomType, id=graphene.ID())

    active_school_term = graphene.Field(SchoolTermType)
    school_terms = FilterOrderList(SchoolTermType)

    holidays = FilterOrderList(HolidayType)
    availability_events = FilterOrderList(AvailabilityEventType)
    my_availability_events = FilterOrderList(AvailabilityEventType)
    calendar = graphene.Field(CalendarBaseType)

    group_types = FilterOrderList(GroupTypeType)

    roles = FilterOrderList(RoleType)

    announcements = FilterOrderList(AnnouncementType)

    oauth_applications = FilterOrderList(OAuthApplicationType)
    init_oauth_application = graphene.Field(InitOAuthApplicationType)
    oauth_scopes = graphene.List(OAuthScopeType)

    object_permissions = graphene.List(
        ObjectPermissionResultType, input=graphene.List(ObjectPermissionInputType)
    )

    countries = graphene.List(CountryType)

    health_check_plugins = graphene.List(HealthCheckPluginType)

    person_invitation_by_code = graphene.Field(PersonInvitationType, code=graphene.String())

    password_help_texts = graphene.List(graphene.String)
    password_validation_status = graphene.List(graphene.String, password=graphene.String())

    dashboard = graphene.Field(DashboardType)

    availability_types = FilterOrderList(AvailabilityTypeType)
    public_availability_types = FilterOrderList(AvailabilityTypeType)

    data_check_results = graphene.List(DataCheckResultType)
    data_checks = graphene.List(DataCheckType)

    password_request_key_verification_status = graphene.Boolean(key=graphene.String())

    def resolve_ping(root, info, payload) -> str:
        return payload

    def resolve_notifications(root, info, **kwargs):
        if info.context.user.is_anonymous:
            return []
        qs = Notification.objects.filter(
            Q(
                pk__in=get_objects_for_user(
                    info.context.user, "core.view_person", Person.objects.all()
                )
            )
            | Q(recipient=info.context.user.person)
        )
        return graphene_django_optimizer.query(qs, info)

    def resolve_persons(root, info, **kwargs):
        qs = get_objects_for_user(info.context.user, "core.view_person", Person.objects.all())
        if has_person(info.context.user):
            qs = qs | Person.objects.filter(id=info.context.user.person.id)

            active_school_term = get_active_school_term(info.context)
            group_q = Q(school_term=active_school_term) | Q(school_term=None)
            qs = qs | Person.objects.filter(
                member_of__in=Group.objects.filter(group_q).filter(
                    owners=info.context.user.person,
                    group_type__owners_can_see_members=True,
                ),
            )
        qs = qs.distinct().annotate_permissions(info.context.user)
        return graphene_django_optimizer.query(qs, info)

    def resolve_person_by_id(root, info, id):  # noqa
        person = Person.objects.get(pk=id)
        if not info.context.user.has_perm("core.view_person_rule", person):
            return None
        return person

    def resolve_person_by_id_or_me(root, info, **kwargs):  # noqa
        # Returns person associated with current user if id is None, else the person with the id
        if "id" not in kwargs or kwargs["id"] is None:
            return info.context.user.person if has_person(info.context.user) else None

        person = Person.objects.get(pk=kwargs["id"])
        if not info.context.user.has_perm("core.view_person_rule", person):
            return None
        return person

    @staticmethod
    def resolve_groups(root, info, **kwargs):
        qs = get_objects_for_user(info.context.user, "core.view_group", Group)
        qs = (
            (
                qs
                | Group.objects.filter(
                    owners=info.context.user.person,
                    group_type__owners_can_see_groups=True,
                )
            )
            .distinct()
            .annotate_permissions(info.context.user)
        )
        qs = filter_active_school_term(info.context, qs)
        return graphene_django_optimizer.query(qs, info)

    @staticmethod
    def resolve_group_by_id(root, info, id):  # noqa
        group = Group.objects.filter(id=id)

        if group.exists():
            group = group.first()

            if not info.context.user.has_perm("core.view_group_rule", group):
                return None
            return group

    @staticmethod
    def resolve_groups_by_owner(root, info, owner=None):
        if owner:
            owner = Person.objects.get(pk=owner)
            if not info.context.user.has_perm("core.view_person_rule", owner):
                return []
        elif has_person(info.context.user):
            owner = info.context.user.person
        else:
            return []

        qs = filter_active_school_term(info.context, owner.owner_of.all())
        return graphene_django_optimizer.query(qs, info)

    def resolve_who_am_i(root, info, **kwargs):
        return info.context.user

    def resolve_system_properties(root, info, **kwargs):
        return True

    def resolve_installed_apps(root, info, **kwargs):
        return [app for app in apps.get_app_configs() if isinstance(app, AppConfig)]

    def resolve_celery_progress_by_id(root, info, id, **kwargs):  # noqa: A002
        task = TaskUserAssignment.objects.get(id=id)

        if not info.context.user.has_perm("core.view_progress_rule", task):
            return None
        progress = task.get_progress_with_meta()
        return progress

    def resolve_celery_progress_by_user(root, info, **kwargs):
        if info.context.user.is_anonymous:
            return None
        tasks = TaskUserAssignment.objects.filter(user=info.context.user)
        return [
            task.get_progress_with_meta()
            for task in tasks
            if task.get_progress_with_meta().get("complete") is False
        ]

    def resolve_pdf_by_id(root, info, id, **kwargs):  # noqa
        pdf_file = PDFFile.objects.get(pk=id)
        if has_person(info.context) and info.context.user.person == pdf_file.person:
            return pdf_file
        return None

    def resolve_search_snippets(root, info, query, limit=-1, **kwargs):
        indexed_models = UnifiedIndex().get_indexed_models()
        allowed_object_ids = get_allowed_object_ids(info.context.user, indexed_models)

        if allowed_object_ids:
            results = (
                SearchQuerySet().filter(id__in=allowed_object_ids).filter(text=AutoQuery(query))
            )
            if limit < 0:
                return results
            return results[:limit]
        else:
            return None

    def resolve_messages(root, info, **kwargs):
        return get_messages(info.context)

    def resolve_current_announcements(root, info, **kwargs):
        if not has_person(info.context.user):
            return []
        return (
            Announcement.objects.relevant_for(info.context.user.person)
            .at_time()
            .order_by("priority")
        )

    def resolve_custom_menu_by_name(root, info, name, **kwargs):
        return CustomMenu.get_default(name)

    def resolve_dynamic_routes(root, info, **kwargs):
        dynamic_routes = []

        for dynamic_route_object in DynamicRoute.registered_objects_dict.values():
            dynamic_routes += dynamic_route_object.get_dynamic_routes()

        return dynamic_routes

    def resolve_two_factor(root, info, **kwargs):
        if info.context.user.is_anonymous:
            return None
        return info.context.user

    @staticmethod
    def resolve_oauth_access_tokens(root, info, **kwargs):
        qs = OAuthAccessToken.objects.filter(user=info.context.user)
        return graphene_django_optimizer.query(qs, info)

    @staticmethod
    def resolve_room_by_id(root, info, **kwargs):
        pk = kwargs.get("id")
        room_object = Room.objects.get(pk=pk)

        if not info.context.user.has_perm("core.view_room_rule", room_object):
            return None

        return room_object

    @staticmethod
    def resolve_availability_events(root, info, **kwargs):
        if info.context.user.has_perm("core.view_availability_events_rule"):
            return get_objects_for_user(
                info.context.user, "core.view_availabilityevent", AvailabilityEvent
            )
        return []

    @staticmethod
    def resolve_my_availability_events(root, info, **kwargs):
        if has_person(info.context.user):
            return AvailabilityEvent.objects.filter(person=info.context.user.person)
        return []

    @staticmethod
    def resolve_active_school_term(root, info, **kwargs):
        return get_active_school_term(info.context)

    @staticmethod
    def resolve_calendar(root, info, **kwargs):
        return True

    @staticmethod
    def resolve_current_school_term(root, info, **kwargs):
        if not has_person(info.context.user):
            return None

        return SchoolTerm.current

    @staticmethod
    def resolve_holidays(root, info, **kwargs):
        qs = get_objects_for_user(info.context.user, "core.view_holiday", Holiday.objects.all())
        return graphene_django_optimizer.query(qs, info)

    @staticmethod
    def resolve_school_terms(root, info, **kwargs):
        if not info.context.user.has_perm("core.fetch_schoolterms_rule"):
            return []
        return graphene_django_optimizer.query(SchoolTerm.objects.all(), info)

    @staticmethod
    def resolve_group_types(root, info, **kwargs):
        if not info.context.user.has_perm("core.fetch_grouptypes_rule"):
            return []
        return graphene_django_optimizer.query(GroupType.objects.all(), info)

    @staticmethod
    def resolve_roles(root, info, **kwargs):
        if info.context.user.has_perm("core.fetch_roles_rule"):
            return graphene_django_optimizer.query(Role.objects.all(), info)
        return []

    @staticmethod
    def resolve_rooms(root, info, **kwargs):
        qs = get_objects_for_user(info.context.user, "core.view_room", Room.objects.all())
        return graphene_django_optimizer.query(qs, info)

    @staticmethod
    def resolve_users(root, info, **kwargs):
        return get_objects_for_user(
            info.context.user, "auth.view_user", get_user_model().objects.all()
        )

    @staticmethod
    def resolve_object_permissions(root, info, input, **kwargs):  # noqa
        results = []

        for object_permission_item in input:
            ct = ContentType.objects.get(
                app_label=object_permission_item.app_label, model=object_permission_item.obj_type
            )
            ct_class = ct.model_class()
            check_obj = ct_class.objects.get(id=object_permission_item.obj_id)

            results.append(
                {
                    "name": object_permission_item.name,
                    "obj_id": object_permission_item.obj_id,
                    "obj_type": object_permission_item.obj_type,
                    "app_label": object_permission_item.app_label,
                    "result": info.context.user.has_perm(object_permission_item.name, check_obj),
                }
            )

        return results

    @staticmethod
    def resolve_announcements(root, info, **kwargs):
        qs = get_objects_for_user(
            info.context.user, "core.view_announcement", Announcement.objects.all()
        )
        return graphene_django_optimizer.query(qs, info)

    @staticmethod
    def resolve_oauth_applications(root, info, **kwargs):
        if not info.context.user.has_perm("core.view_oauthapplication_rule"):
            return []
        return graphene_django_optimizer.query(OAuthApplication.objects.all(), info)

    @staticmethod
    def resolve_init_oauth_application(root, info, **kwargs):
        if not info.context.user.has_perm("core.create_oauthapplication_rule"):
            return None
        return InitOAuthApplicationType(
            client_id=generate_client_id(), client_secret=generate_client_secret()
        )

    @staticmethod
    def resolve_oauth_scopes(root, info, **kwargs):
        if not (
            info.context.user.has_perm("core.view_oauthapplication_rule")
            | info.context.user.has_perm("core.create_oauthapplication_rule")
        ):
            return []

        return [
            OAuthScopeType(name=key, description=value)
            for key, value in AppScopes().get_all_scopes().items()
        ]

    @staticmethod
    def resolve_countries(root, info, **kwargs):
        return countries

    @staticmethod
    def resolve_health_check_plugins(root, info, **kwargs):
        if not info.context.user.has_perm("core.view_system_status_rule"):
            return []

        # This is taken directly from django-health-check
        registering_plugins = (
            plugin_class(**copy.deepcopy(options)) for plugin_class, options in plugin_dir._registry
        )
        registering_plugins = sorted(registering_plugins, key=lambda plugin: plugin.identifier())

        return registering_plugins

    @staticmethod
    def resolve_celery_inspect_task_results(root, info, **kwargs):
        if not info.context.user.has_perm("core.view_system_status_rule"):
            return []

        task_results = []

        if celery_app.control.inspect().registered_tasks():
            job_list = list(celery_app.control.inspect().registered_tasks().values())[0]
            for job in job_list:
                job_task_results = TaskResult.objects.filter(task_name=job)
                if job_task_results.exists():
                    task_results.append(job_task_results.order_by("date_done").last())

        return task_results

    @staticmethod
    def resolve_person_invitation_by_code(root, info, code, **kwargs):
        formatted_code = "".join(code.lower().split("-"))
        if PersonInvitation.objects.filter(key=formatted_code).exists():
            return PersonInvitation.objects.get(key=formatted_code)
        return None

    @staticmethod
    def resolve_password_help_texts(root, info, **kwargs):
        return password_validators_help_texts()

    @staticmethod
    def resolve_password_validation_status(root, info, password, **kwargs):
        try:
            validate_password(password, info.context.user)
            return []
        except ValidationError as exc:
            return exc.messages

    @staticmethod
    def resolve_dashboard(root, info, **kwargs):
        return True

    @staticmethod
    def resolve_availability_types(root, info, **kwargs):
        if info.context.user.has_perm("core.view_all_availability_types_rule"):
            return AvailabilityType.objects.all()
        elif info.context.user.has_perm("core.view_public_availability_types_rule"):
            return AvailabilityType.objects.filter(public=True)
        return []

    @staticmethod
    def resolve_public_availability_types(root, info, **kwargs):
        if info.context.user.has_perm("core.view_public_availability_types_rule"):
            return AvailabilityType.objects.filter(public=True)
        return []

    @staticmethod
    def resolve_data_check_results(root, info, **kwargs):
        if not info.context.user.has_perm("core.view_datacheckresults_rule"):
            return []

        return DataCheckResult.objects.filter(
            content_type__app_label__in=apps.app_configs.keys()
        ).order_by("data_check")

    @staticmethod
    def resolve_data_checks(root, info, **kwargs):
        if not info.context.user.has_perm("core.view_datacheckresults_rule"):
            return []

        return DataCheck.registered_objects_list

    @staticmethod
    def resolve_password_request_key_verification_status(*args, **kwargs):
        return check_password_reset_key(*args, **kwargs)


class Mutation(graphene.ObjectType):
    delete_persons = PersonBatchDeleteMutation.Field()
    create_persons = PersonBatchCreateMutation.Field()
    update_persons = PersonBatchPatchMutation.Field()

    change_user_password = ChangePasswordMutation.Field()
    request_password_reset = RequestPasswordResetMutation.Field()

    mark_notification_read = MarkNotificationReadMutation.Field()
    mark_all_notifications_read = MarkAllNotificationsReadMutation.Field()

    celery_progress_fetched = CeleryProgressFetchedMutation.Field()

    revoke_oauth_tokens = OAuthBatchRevokeTokenMutation.Field()

    create_rooms = RoomBatchCreateMutation.Field()
    delete_rooms = RoomBatchDeleteMutation.Field()
    update_rooms = RoomBatchPatchMutation.Field()

    create_school_terms = SchoolTermBatchCreateMutation.Field()
    delete_school_terms = SchoolTermBatchDeleteMutation.Field()
    update_school_terms = SchoolTermBatchPatchMutation.Field()
    set_active_school_term = SetActiveSchoolTermMutation.Field()

    create_holidays = HolidayBatchCreateMutation.Field()
    delete_holidays = HolidayBatchDeleteMutation.Field()
    update_holidays = HolidayBatchPatchMutation.Field()

    create_availability_events = AvailabilityEventBatchCreateMutation.Field()
    delete_availability_events = AvailabilityEventBatchDeleteMutation.Field()
    update_availability_events = AvailabilityEventBatchPatchMutation.Field()

    create_availability_types = AvailabilityTypeBatchCreateMutation.Field()
    delete_availability_types = AvailabilityTypeBatchDeleteMutation.Field()
    update_availability_types = AvailabilityTypeBatchPatchMutation.Field()

    create_personal_events = PersonalEventBatchCreateMutation.Field()
    delete_personal_events = PersonalEventBatchDeleteMutation.Field()
    update_personal_events = PersonalEventBatchPatchMutation.Field()

    create_personal_todos = PersonalTodoBatchCreateMutation.Field()
    delete_personal_todos = PersonalTodoBatchDeleteMutation.Field()
    update_personal_todos = PersonalTodoBatchPatchMutation.Field()

    set_calendar_status = SetCalendarStatusMutation.Field()

    create_group_types = GroupTypeBatchCreateMutation.Field()
    delete_group_types = GroupTypeBatchDeleteMutation.Field()
    update_group_types = GroupTypeBatchPatchMutation.Field()

    create_roles = RoleBatchCreateMutation.Field()
    delete_roles = RoleBatchDeleteMutation.Field()
    update_roles = RoleBatchPatchMutation.Field()

    delete_groups = GroupBatchDeleteMutation.Field()

    create_announcements = AnnouncementBatchCreateMutation.Field()
    delete_announcements = AnnouncementBatchDeleteMutation.Field()
    patch_announcements = AnnouncementBatchPatchMutation.Field()

    create_oauth_applications = OAuthApplicationBatchCreateMutation.Field()
    delete_oauth_applications = OAuthApplicationBatchDeleteMutation.Field()
    patch_oauth_applications = OAuthApplicationBatchPatchMutation.Field()

    set_maintenance_mode = SetMaintenanceModeMutation.Field()

    send_account_registration = SendAccountRegistrationMutation.Field()

    create_dashboard_widgets = CreateDashboardWidgetsMutation.Field()
    reorder_dashboard_widgets = ReorderDashboardWidgetsMutation.Field()
    delete_dashboard_widget_instances = DashboardWidgetInstanceDeleteMutation.Field()
    delete_dashboard_widgets = DashboardWidgetDeleteMutation.Field()
    update_dashboard_widgets = DashboardWidgetUpdateMutation.Field()

    update_calendar_widgets = CalendarWidgetBatchPatchMutation.Field()
    update_external_link_widgets = ExternalLinkWidgetBatchPatchMutation.Field()
    update_static_content_widgets = StaticContentWidgetBatchPatchMutation.Field()

    solve_data_check_result = SolveDataCheckResultMutation.Field()
    run_data_checks = RunDataChecksMutation.Field()

    set_todo_completed = SetTodoCompletedMutation.Field()

    activate_totp = ActivateTOTPMutation.Field()
    deactivate_authenticator = DeactivateAuthenticatorMutation.Field()
    generate_recovery_codes = GenerateRecoveryCodesMutation.Field()
    add_security_key = AddSecurityKeyMutation.Field()


types = [*dashboard_types]


def build_global_schema():
    """Build global GraphQL schema from all apps."""
    query_bases = [Query]
    mutation_bases = [Mutation]
    additional_types = types

    for app in get_app_packages():
        schema_mod = get_app_module(app, "schema")
        if not schema_mod:
            # The app does not define a schema
            continue

        if AppQuery := getattr(schema_mod, "Query", None):
            query_bases.append(AppQuery)
        if AppMutation := getattr(schema_mod, "Mutation", None):
            mutation_bases.append(AppMutation)
        if app_types := getattr(schema_mod, "types", None):
            additional_types.extend(app_types)

    # Define classes using all query/mutation classes as mixins
    #  cf. https://docs.graphene-python.org/projects/django/en/latest/schema/#adding-to-the-schema
    GlobalQuery = type("GlobalQuery", tuple(query_bases), {})
    GlobalMutation = type("GlobalMutation", tuple(mutation_bases), {})

    return graphene.Schema(query=GlobalQuery, mutation=GlobalMutation, types=additional_types)


schema = build_global_schema()



# Generated by Django 5.1.6 on 2025-03-17 17:39

import aleksis.core.mixins
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


def migrate_guardians(apps, schema_editor):
    PersonModel = apps.get_model('core', 'Person')
    RoleModel = apps.get_model('core', 'Role')
    PersonRelationshipModel = apps.get_model('core', 'PersonRelationship')
    GlobalPreferenceModel = apps.get_model("dynamic_preferences", "GlobalPreferenceModel")

    db_alias = schema_editor.connection.alias

    guardian_role, _ = RoleModel.objects.get_or_create(
        managed_by_app_label="core",
        short_name="guardian",
        defaults={"name": "Guardian", "reciprocal_name": "Child"},
    )

    pref = GlobalPreferenceModel.objects.get_or_create(
        section="people",
        name="roles",
        defaults={"raw_value": str(guardian_role.pk)}
    )

    for person in PersonModel.objects.filter(guardians__isnull=False):
        for guardian in person.guardians.all():
            relationship = PersonRelationshipModel.objects.create(
                person=guardian,
                of_person=person,
            )
            relationship.roles.add(guardian_role)


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('core', '0078_add_role'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]


    operations = [
        migrations.CreateModel(
            name='PersonRelationship',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('managed_by_app_label', models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='personrelationship',
            name='of_person',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='core.person'),
        ),
        migrations.AddField(
            model_name='personrelationship',
            name='person',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='core.person'),
        ),
        migrations.AddField(
            model_name='personrelationship',
            name='roles',
            field=models.ManyToManyField(blank=True, related_name='+', to='core.role', verbose_name='Roles'),
        ),
        migrations.AddField(
            model_name='person',
            name='related_persons',
            field=models.ManyToManyField(blank=True, through='core.PersonRelationship', related_name="related_to_persons", to='core.person', verbose_name='Related persons'),
        ),
        migrations.RunPython(migrate_guardians),
    ]



import graphene
from graphene_django import DjangoObjectType

from aleksis.core.models import OAuthAccessToken, OAuthApplication

from .base import (
    BaseBatchCreateMutation,
    BaseBatchDeleteMutation,
    BaseBatchPatchMutation,
    FieldFileType,
    PermissionsTypeMixin,
)


class OAuthScopeType(graphene.ObjectType):
    name = graphene.String()
    description = graphene.String()


class OAuthApplicationType(PermissionsTypeMixin, DjangoObjectType):
    class Meta:
        model = OAuthApplication
        fields = [
            "id",
            "name",
            "icon",
            "client_id",
            "client_secret",
            "client_type",
            "algorithm",
            "allowed_scopes",
            "redirect_uris",
            "skip_authorization",
        ]

    icon = graphene.Field(FieldFileType)

    @staticmethod
    def resolve_algorithm(root, info, **kwargs):
        """graphene-django-cud has the undocumented behavior to use "A_" instead of the empty
        string"""
        if not root.algorithm:
            return "A_"
        else:
            return root.algorithm

    @staticmethod
    def resolve_client_secret(root, info, **kwargs):
        """Always return empty client secret as hashed client secret is no useful information."""
        return ""


class InitOAuthApplicationType(graphene.ObjectType):
    client_id = graphene.String()
    client_secret = graphene.String()


class OAuthAccessTokenType(DjangoObjectType):
    scopes = graphene.List(OAuthScopeType)

    @staticmethod
    def resolve_scopes(root: OAuthAccessToken, info, **kwargs):
        return [OAuthScopeType(name=key, description=value) for key, value in root.scopes.items()]

    class Meta:
        model = OAuthAccessToken
        fields = ["id", "application", "expires", "created", "updated"]


class OAuthBatchRevokeTokenMutation(graphene.Mutation):
    class Arguments:
        ids = graphene.List(graphene.ID)

    ok = graphene.Boolean()

    @staticmethod
    def mutate(root, info, ids):
        OAuthAccessToken.objects.filters(pk__in=ids, user=info.context.user).delete()
        return len(ids)


class OAuthApplicationBatchCreateMutation(BaseBatchCreateMutation):
    class Meta:
        model = OAuthApplication
        permissions = ("core.create_oauthapplication_rule",)
        only_fields = (
            "name",
            "icon",
            "client_id",
            "client_secret",
            "client_type",
            "algorithm",
            "allowed_scopes",
            "redirect_uris",
            "skip_authorization",
        )


class OAuthApplicationBatchDeleteMutation(BaseBatchDeleteMutation):
    class Meta:
        model = OAuthApplication
        permissions = ("core.delete_oauthapplication_rule",)


class OAuthApplicationBatchPatchMutation(BaseBatchPatchMutation):
    class Meta:
        model = OAuthApplication
        permissions = ("core.edit_oauthapplication_rule",)
        only_fields = (
            "id",
            "name",
            "icon",
            "client_type",
            "algorithm",
            "allowed_scopes",
            "redirect_uris",
            "skip_authorization",
        )



from collections.abc import Generator, Iterable, Sequence
from functools import wraps
from numbers import Number
from typing import Callable, Optional, Union

from django.contrib import messages

from celery_progress.backend import PROGRESS_STATE, AbstractProgressRecorder

from ..celery import app
from ..tasks import send_notification_for_done_task


class ProgressRecorder(AbstractProgressRecorder):
    """Track the progress of a Celery task and give data to the frontend.

    This recorder provides the functions `set_progress` and `add_message`
    which can be used to track the status of a Celery task.

    How to use
    ----------
    1. Write a function and include tracking methods

    ::

        from django.contrib import messages

        from aleksis.core.util.celery_progress import recorded_task

        @recorded_task
        def do_something(foo, bar, recorder, baz=None):
            # ...
            recorder.set_progress(total=len(list_with_data))

            for i, item in enumerate(list_with_data):
                # ...
                recorder.set_progress(i + 1)
                # ...

            recorder.add_message(messages.SUCCESS, "All data were imported successfully.")

    You can also use `recorder.iterate` to simplify iterating and counting.

    2. Track progress:

    ::

        # Render progress view
        do_something.delay_with_progress(foo, bar, baz=baz, TaskUserAssignment(
            user=...,
            title=_("Progress: Import data"),
            back_url=reverse("index"),
            progress_title=_("Import objects …"),
            success_message=_("The import was done successfully."),
            error_message=_("There was a problem while importing data."),
        ))

    Please take a look at the documentation of ``TaskUserAssignment``
    to get all available options.
    """

    def __init__(self, task):
        self.task = task
        self._messages = []
        self._current = 0
        self._total = 100
        self.set_progress()

    def iterate(self, data: Union[Iterable, Sequence], total: Optional[int] = None) -> Generator:
        """Iterate over a sequence or iterable, updating progress on the move.

        ::

            @recorded_task
            def do_something(long_list, recorder):
                for item in recorder.iterate(long_list):
                    do_something_with(item)

        :param data: A sequence (tuple, list, set,...) or an iterable
        :param total: Total number of items, in case data does not support len()
        """
        if total is None and hasattr(data, "__len__"):
            total = len(data)
        else:
            raise TypeError("No total value passed, and data does not support len()")

        for current, item in enumerate(data):
            self.set_progress(current, total)
            yield item

    def set_progress(
        self,
        current: Optional[Number] = None,
        total: Optional[Number] = None,
        description: Optional[str] = None,
        level: int = messages.INFO,
    ):
        """Set the current progress in the frontend.

        The progress percentage is automatically calculated in relation to self.total.

        :param current: The number of processed items; relative to total, default unchanged
        :param total: The total number of items (or 100 if using a percentage), default unchanged
        :param description: A textual description, routed to the frontend as an INFO message
        """
        if current is not None:
            self._current = current
        if total is not None:
            self._total = total

        percent = 0
        if self._total > 0:
            percent = self._current / self._total * 100

        if description is not None:
            self._messages.append((level, description))

        self.task.update_state(
            state=PROGRESS_STATE,
            meta={
                "current": self._current,
                "total": self._total,
                "percent": percent,
                "messages": self._messages,
            },
        )

    def add_message(self, level: int, message: str) -> None:
        """Show a message in the progress frontend.

        This method is a shortcut for set_progress with no new progress arguments,
        passing only the message and level as description.

        :param level: The message level (default levels from django.contrib.messages)
        :param message: The actual message (should be translated)
        """
        self.set_progress(description=message, level=level)


def recorded_task(orig: Optional[Callable] = None, **kwargs) -> Union[Callable, app.Task]:
    """Create a Celery task that receives a ProgressRecorder.

    Returns a Task object with a wrapper that passes the recorder instance
    as the recorder keyword argument.
    """

    def _real_decorator(orig: Callable) -> app.Task:
        @wraps(orig)
        def _inject_recorder(task, *args, **kwargs):
            recorder = ProgressRecorder(task)
            orig(*args, **kwargs, recorder=recorder)

            # Start notification task to ensure
            # that the user is informed about the result in any case
            send_notification_for_done_task.delay(task.request.id)

            return recorder._messages

        # Force bind to True because _inject_recorder needs the Task object
        kwargs["bind"] = True
        return app.task(_inject_recorder, **kwargs)

    if orig and not kwargs:
        return _real_decorator(orig)
    return _real_decorator



# Generated by Django 5.0.6 on 2024-07-12 11:34

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('core', '0068_calendar_event_amends_unique_constraints'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='CalendarAlarm',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('managed_by_app_label', models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance')),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('action', models.CharField(choices=[('audio', 'Audio'), ('display', 'Display'), ('email', 'Email'), ('procedure', 'Procedure')], default='display', max_length=10, verbose_name='Action')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='alarms', to='core.calendarevent', verbose_name='Event')),
                ('send_notifications', models.BooleanField(default=False, verbose_name='Send notifications')),
                ('polymorphic_ctype', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_%(app_label)s.%(class)s_set+', to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name': 'Calendar alarm',
                'verbose_name_plural': 'Calendar alarms',
            },
        ),
        migrations.CreateModel(
            name='PersonalEventAlarm',
            fields=[
                ('calendaralarm_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.calendaralarm')),
            ],
            options={
                'verbose_name': 'Personal event alarm',
                'verbose_name_plural': 'Personal event alarms',
            },
            bases=('core.calendaralarm',),
        ),
        migrations.AddField(
            model_name='notification',
            name='calendar_alarm',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to='core.calendaralarm', verbose_name='Calendar alarm'),
        ),
        migrations.AddConstraint(
            model_name='notification',
            constraint=models.UniqueConstraint(condition=models.Q(('calendar_alarm__isnull', False)), fields=('calendar_alarm', 'recipient'), name='unique_recipient_per_calendar_alarm'),
        ),
    ]



# Generated by Django 5.1.5 on 2025-05-09 17:29

import aleksis.core.mixins
import ckeditor.fields
import colorfield.fields
import django.contrib.postgres.fields
import django.db.models.deletion
import uuid
from django.conf import settings
from django.db import migrations, models

def migrate_availability_events(apps, schema_editor):
    AvailabilityEvent = apps.get_model("core", "AvailabilityEvent")
    AvailabilityType = apps.get_model("core", "AvailabilityType")

    free_availability_type = AvailabilityType.objects.create(name="Free", short_name="f", free=True, public=True, managed_by_app_label="core")
    busy_availability_type = AvailabilityType.objects.create(name="Busy", short_name="b", free=False, public=True, managed_by_app_label="core")

    for a in AvailabilityEvent.objects.all():
        if a.free:
            a.availability_type = free_availability_type
        else:
            a.availability_type = busy_availability_type
        a.save()


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0087_add_uuid'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AvailabilityType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('managed_by_app_label', models.CharField(blank=True, editable=False, max_length=255, verbose_name='App label of app responsible for managing this instance')),
                ('uuid', models.UUIDField(db_default=models.Func(function='gen_random_uuid'), editable=False)),
                ('extended_data', models.JSONField(default=dict, editable=False)),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('short_name', models.CharField(max_length=100, verbose_name='Short name')),
                ('description', models.TextField(blank=True, verbose_name='Description')),
                ('color', colorfield.fields.ColorField(blank=True, null=True, default='', image_field=None, max_length=25, samples=None, verbose_name='Color')),
                ('free', models.BooleanField(help_text='This indicates whether availability events with this type should count as free.', verbose_name='Free')),
                ('public', models.BooleanField(help_text='This indicates whether this type should be publicly available.', verbose_name='Public')),
            ],
            options={
                'verbose_name': 'Availability type',
                'verbose_name_plural': 'Availability types',
            },
        ),
        migrations.AddField(
            model_name='availabilityevent',
            name='availability_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='availability_events', to='core.availabilitytype', verbose_name='Availability type'),
        ),
        migrations.RunPython(migrate_availability_events),
        migrations.RemoveField(
            model_name='availabilityevent',
            name='free',
        ),
        migrations.AlterField(
            model_name='availabilityevent',
            name='availability_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='availability_events', to='core.availabilitytype', verbose_name='Availability type'),
        ),
    ]



import random

from django.core.exceptions import PermissionDenied
from django.db.models.query import QuerySet

import graphene
from graphene.types.objecttype import ObjectType
from graphene_django.registry import get_global_registry

from aleksis.core.models import (
    DashboardWidget,
    DashboardWidgetInstance,
)
from aleksis.core.schema.base import (
    BaseBatchDeleteMutation,
    BaseBatchPatchMutation,
    BaseObjectType,
    DjangoFilterMixin,
    PermissionsTypeMixin,
)
from aleksis.core.util.core_helpers import has_person


class DashboardWidgetObjectType(BaseObjectType):
    class Meta:
        model = DashboardWidget


class DashboardWidgetType(PermissionsTypeMixin, DjangoFilterMixin, graphene.Interface):
    class Meta:
        model = DashboardWidget

    id = graphene.ID()
    status = graphene.Field(graphene.Enum.from_enum(DashboardWidget.Status))
    title = graphene.String()


class DashboardWidgetInstanceType(BaseObjectType):
    class Meta:
        model = DashboardWidgetInstance
        fields = ["id", "x", "y", "width", "height"]

    context = graphene.JSONString()
    widget = graphene.Field(DashboardWidgetType)

    @staticmethod
    def resolve_context(root, info, **kwargs):
        return root.widget.get_context(info.context, root.configuration)


class AvailableDashboardWidgetType(ObjectType):
    model_name = graphene.String()
    type_name = graphene.String()

    @staticmethod
    def resolve_type_name(root, info, **kwargs):
        return get_global_registry().get_type_for_model(root)._meta.name

    @staticmethod
    def resolve_model_name(root, info, **kwargs):
        return root._class_name


class DashboardType(ObjectType):
    widgets = graphene.List(
        DashboardWidgetType,
        status=graphene.Argument(graphene.Enum.from_enum(DashboardWidget.Status)),
    )
    widget_types = graphene.List(AvailableDashboardWidgetType)

    my = graphene.List(
        DashboardWidgetInstanceType,
        force_default=graphene.Argument(graphene.Boolean),
    )
    has_own = graphene.Boolean()

    @staticmethod
    def resolve_widgets(root, info, status=None, **kwargs) -> QuerySet[DashboardWidget]:
        if info.context.user.has_perm("core.view_dashboardwidgets_rule"):
            if status is not None:
                return DashboardWidget.objects.filter(status=status).order_by("-id")
            return DashboardWidget.objects.all().order_by("-id")
        return DashboardWidget.objects.none()

    @staticmethod
    def resolve_widget_types(root, info, **kwargs):
        return DashboardWidget.registered_objects_list

    @staticmethod
    def resolve_single_widgets(root, info, **kwargs):
        return True

    @staticmethod
    def resolve_my(root, info, force_default=False, **kwargs):
        if not force_default and has_person(info.context.user):
            qs = DashboardWidgetInstance.objects.for_person(info.context.user.person)
            if qs.exists():
                return qs.order_by("y", "x")
        return DashboardWidgetInstance.objects.default_dashboard().order_by("y", "x")

    @staticmethod
    def resolve_has_own(root, info, **kwargs):
        return (
            has_person(info.context.user)
            and DashboardWidgetInstance.objects.for_person(info.context.user.person).exists()
        )


class ReorderDashboardWidgetInput(graphene.InputObjectType):
    id = graphene.ID(required=False)
    widget = graphene.ID(required=True)
    x = graphene.Int(required=True)
    y = graphene.Int(required=True)
    width = graphene.Int(required=True)
    height = graphene.Int(required=True)


class ReorderDashboardWidgetsMutation(graphene.Mutation):
    class Arguments:
        widget_data = graphene.List(ReorderDashboardWidgetInput, required=True)

        is_default_dashboard = graphene.Boolean(required=False)

    created = graphene.List(DashboardWidgetInstanceType)
    updated = graphene.List(DashboardWidgetInstanceType)

    @classmethod
    def mutate(
        cls,
        root,
        info,
        widget_data: list[ReorderDashboardWidgetInput],
        is_default_dashboard=False,
    ):
        if is_default_dashboard and not info.context.user.has_perm(
            "core.edit_default_dashboard_rule"
        ):
            raise PermissionDenied

        person = None if is_default_dashboard else info.context.user.person

        created = []
        updated = []

        for w_data in widget_data:
            if w_data.id:
                widget = DashboardWidgetInstance.objects.get(id=w_data.id)
                updated.append(widget)
            else:
                widget = DashboardWidgetInstance()
                widget.widget = DashboardWidget.objects.get(pk=w_data.widget)
                created.append(widget)

            widget.x = w_data.x
            widget.y = w_data.y
            widget.width = w_data.width
            widget.height = w_data.height
            widget.person = person

            widget.save()

        return ReorderDashboardWidgetsMutation(created=created, updated=updated)


class CreateDashboardWidgetsMutation(graphene.Mutation):
    class Arguments:
        widget_types = graphene.List(graphene.String, required=True)

    created = graphene.List(DashboardWidgetType)

    @classmethod
    def mutate(cls, root, info, widget_types: list[str], **kwargs):
        created = []
        for type_name in widget_types:
            widget_class = DashboardWidget.get_object_by_name(type_name)

            if not widget_class:
                raise ValueError(f"Invalid dashboard widget subclass {type_name}")

            widget = widget_class()
            widget.title = f"{widget_class._meta.verbose_name} {random.randint(1000, 9999)}"  # noqa: S311
            widget.save()

            created.append(widget)

        return CreateDashboardWidgetsMutation(created=created)


class DashboardWidgetInstanceDeleteMutation(BaseBatchDeleteMutation):
    class Meta:
        model = DashboardWidgetInstance
        permissions = ("core.delete_dashboard_widget_instance_rule",)


class DashboardWidgetUpdateMutation(BaseBatchPatchMutation):
    class Meta:
        model = DashboardWidget
        permissions = ("core.edit_dashboardwidget_rule",)
        fields = (
            "id",
            "status",
            "title",
        )
        return_field_name = "dashboardWidgets"

    @classmethod
    def after_mutate(cls, root, info, input, updated_objs, return_data):  # noqa: A002
        return_data["dashboardWidgets"] = [obj.dashboardwidget_ptr for obj in updated_objs]


class DashboardWidgetDeleteMutation(BaseBatchDeleteMutation):
    class Meta:
        model = DashboardWidget
        permissions = ("core.delete_dashboardwidget_rule",)



# Generated by Django 5.1.1 on 2024-10-13 17:14

import django.contrib.postgres.fields
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0065_calendarevent_update_timezone_constraints'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='grouptype',
            name='owners_can_see_groups',
            field=models.BooleanField(default=False, verbose_name='Owners of groups with this group type can see the groups'),
        ),
        migrations.AddField(
            model_name='grouptype',
            name='owners_can_see_members',
            field=models.BooleanField(default=False, verbose_name='Owners of groups with this group type can see group members'),
        ),
        migrations.AddField(
            model_name='grouptype',
            name='owners_can_see_members_allowed_information',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(blank=True, choices=[('personal_details', 'Personal details'), ('address', 'Address'), ('contact_details', 'Contact details'), ('photo', 'Photo'), ('avatar', 'Avatar'), ('groups', 'Groups')], max_length=255), blank=True, default=[], size=None, verbose_name="Information owners of groups with this group type can see of the group's members"),
        ),
    ]



from django.core.exceptions import PermissionDenied

import graphene
from maintenance_mode.core import set_maintenance_mode


class SetMaintenanceModeMutation(graphene.Mutation):
    class Arguments:
        mode = graphene.Boolean()

    mode = graphene.Boolean()

    @classmethod
    def mutate(cls, root, info, mode):
        if not info.context.user.has_perm("core.set_maintenance_mode_rule"):
            raise PermissionDenied()

        set_maintenance_mode(mode)

        return SetMaintenanceModeMutation(mode=mode)
